"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InOrderSegmentTree = void 0;
const u32_1 = require("../math/u32");
const arrayUtils_1 = require("src/collection/arrayUtils");
/*
    mca(a, b) = lsp(a | msp(b - a)) ; // where a <= b
    left(i) = i - (lsp(i + 1) >>> 1)
    right(i) = i + (lsp(i + 1) >>> 1)
    parent(i) =
        offset = lsp(i + 1)
        i + offset - ((i & 2*offset) >>> 0)
*/
/**
 * A {@link SegmentTree} with entries stored in in-order traversal.
 * Inspired by [Tristan Hume's IForestIndex](https://thume.ca/2021/03/14/iforests) ([github](https://github.com/trishume/gigatrace))
 */
class InOrderSegmentTree {
    /**
     * Construct a new segment tree
     *
     * @param combine - The function used to aggregate segment information
     * @param elements - A set of elements to add into the initial tree
     */
    constructor(combine, elements = []) {
        this.array = [];
        this.combine = combine;
        this.build(elements);
    }
    clear() {
        this.array.length = 0;
    }
    pop() {
        // Sanitize range
        if (this.size < 1) {
            return undefined;
        }
        // Un-complete aggregation nodes
        const i = this.array.length - 1;
        for (let mask = 2; i & mask; mask *= 2) {
            this.array[i - mask] = this.array[i - mask - (mask >>> 1)];
        }
        // Return element
        const out = this.array[i - 1];
        this.array.length -= 2;
        return out;
    }
    push(element) {
        // Sanitize range
        if (this.size >= InOrderSegmentTree.MAX_SIZE) {
            throw new RangeError(`Invalid length`);
        }
        // Add element
        const i = this.array.length;
        this.array[i + 1] = this.set(i, element);
        return this.size;
    }
    query(min, max) {
        // Sanitize range
        if (min >= max) {
            throw new RangeError(`Range [${min}..${max}) is empty`);
        }
        if (min < 0 || max > this.size) {
            throw new RangeError(`Range [${min}..${max}) not in [0..${this.size})`);
        }
        // Translate range to interior indices
        min *= 2;
        max *= 2;
        // Jump to min's highest aggregation node that is fully within the range
        let offset = u32_1.lsp(min | u32_1.msp(max - min));
        let value = this.array[min - 1 + (offset >>> 1)];
        // Continue jumping aggregation nodes until max is reached
        for (min += offset; min < max; min += offset) {
            offset = u32_1.lsp(min | u32_1.msp(max - min));
            value = this.combine(value, this.array[min - 1 + (offset >>> 1)]);
        }
        return value;
    }
    get size() {
        return this.array.length >>> 1;
    }
    /**
     * Return an iterator through the tree's elements
     */
    *[Symbol.iterator]() {
        for (let i = 0; i < this.array.length; i += 2) {
            yield this.array[i];
        }
    }
    update(min, max, operation) {
        // Sanitize range
        if (min >= max) {
            return;
        }
        if (min < 0 || max > this.size) {
            throw new RangeError(`Range [${min}..${max}) not in [0..${this.size})`);
        }
        // Translate range to interior indices
        min *= 2;
        max *= 2;
        // Update the values
        let value;
        do {
            value = this.set(min, operation(this.array[min], min >>> 1));
            min += 2;
        } while (min < max);
        // Update remaining aggregation nodes
        let dc = 0;
        let dp = u32_1.lsp(min);
        max = u32_1.msb(min ^ this.array.length) - u32_1.lsb(min);
        for (--min; max > 0; --max) {
            value = this.combine(value, this.array[min + (dp >>> 1) - dc]);
            this.array[min] = value;
            dc = (min & (2 * dp)) >>> 0;
            min += dp - dc;
            dp *= 2;
        }
        // Update the incomplete aggregation node
        this.array[min] = value;
    }
    /**
     * A helper method used to build the tree
     *
     * @param elements The initial set of elements to add into the tree
     */
    build(elements) {
        for (const element of elements) {
            this.push(element);
        }
    }
    /**
     * A helper method to update complete aggregation nodes for an index
     */
    set(index, element) {
        // Set the index
        this.array[index++] = element;
        // Update complete aggregation nodes, from lowest to highest
        for (let mask = 2; index & mask; mask *= 2) {
            element = this.combine(this.array[index - mask - (mask >>> 1)], element);
            this.array[index - mask] = element;
        }
        return element;
    }
}
exports.InOrderSegmentTree = InOrderSegmentTree;
/**
 * The maximum amount of elements that can be added.
 *
 * n elements require 2n memory.
 */
InOrderSegmentTree.MAX_SIZE = arrayUtils_1.MAX_ARRAY_LENGTH >>> 1;
//# sourceMappingURL=inOrderSegmentTree.js.map